@@ -42,6 +42,8 @@ class QScrollEvent;
 class QScrollPrepareEvent;
 class KMessageWidget;
 
+struct UBiDi;
+
 namespace Konsole
 {
 class TerminalPainter;
@@ -340,11 +342,6 @@ public:
         return _semanticUpDown;
     }
 
-    int semanticHints() const
-    {
-        return _semanticHints;
-    }
-
     ColorSchemeWallpaper::Ptr wallpaper() const
     {
         return _wallpaper;
@@ -398,6 +395,31 @@ public:
     // Used to show/hide the message widget
     void updateReadOnlyState(bool readonly);
 
+    void setSelectMode(bool readonly);
+
+    bool getReadOnly() const
+    {
+        return _readOnly;
+    }
+
+    // Get mapping between visual and logical positions in line
+    // returns the index of the last non space character.
+    int bidiMap(Character *screenline,
+                QString &line,
+                int *log2line,
+                int *line2log,
+                uint16_t *shapemap,
+                int32_t *vis2line,
+                bool &shaped,
+                bool shape = true,
+                bool bidi = true) const;
+
+    void showNotification(QString text);
+
+    //
+    // Clear mouse selection, but not keyboard selection
+    void clearMouseSelection();
+
 public Q_SLOTS:
     /**
      * Causes the terminal display to fetch the latest character image from the associated
@@ -669,6 +691,8 @@ private:
     bool _resizing = false;
     bool _showTerminalSizeHint = true;
     bool _bidiEnabled = false;
+    bool _bidiLineLTR = true;
+    bool _bidiTableDirOverride = true;
     bool _usesMouseTracking = false;
     bool _allowMouseTracking = true;
     bool _bracketedPasteMode = false;
@@ -774,9 +798,13 @@ private:
 
     std::unique_ptr<KonsolePrintManager> _printManager;
 
-    int _semanticHints;
     bool _semanticUpDown;
     bool _semanticInputClick;
+
+    UBiDi *ubidi = nullptr;
+
+    int _selModeModifiers;
+    bool _selModeByModifiers; // Selection started by Shift+Arrow
 };
 
 }
