@@ -38,10 +38,17 @@ public:
     int fontHeight() const;
     int fontWidth() const;
     int fontAscent() const;
+    int lineWidth() const;
+    int underlinePos() const;
+    int strikeOutPos() const;
+    int overlinePos() const;
     bool boldIntense() const;
     bool antialiasText() const;
     bool useFontLineCharacters() const;
 
+    bool hasExtraFont(int i) const;
+    QFont getExtraFont(int i) const;
+
 protected:
     void fontChange(const QFont &);
 
@@ -51,9 +58,14 @@ private:
     int m_fontHeight = 1;
     int m_fontWidth = 1;
     int m_fontAscent = 1;
+    int m_lineWidth = 1;
+    int m_underlinePos = 1;
+    int m_strikeOutPos = 1;
+    int m_overlinePos = 1;
     bool m_boldIntense = false;
     bool m_antialiasText = true;
     bool m_useFontLineCharacters = false;
+    QMap<int, QFont> extraFonts;
 
     Profile::Ptr m_profile;
 };
