@@ -14,7 +14,7 @@
 #include <QFileDialog>
 #include <QIcon>
 #include <QInputDialog>
-#include <QPainter>
+#include <QMenu>
 #include <QPushButton>
 #include <QRegularExpressionValidator>
 #include <QStandardItem>
@@ -28,11 +28,11 @@
 #include <KCodecAction>
 #include <KIconDialog>
 #include <KLocalizedString>
-#include <KMessageBox>
-#include <KNSCore/Engine>
 #include <KWindowSystem>
 #include <kconfigwidgets_version.h>
 
+#include <KNSWidgets/Button>
+
 // Konsole
 #include "ui_EditProfileAdvancedPage.h"
 #include "ui_EditProfileAppearancePage.h"
@@ -55,23 +55,10 @@ using namespace Konsole;
 
 EditProfileDialog::EditProfileDialog(QWidget *parent)
     : KPageDialog(parent)
-    , _generalUi(nullptr)
-    , _tabsUi(nullptr)
-    , _appearanceUi(nullptr)
-    , _scrollingUi(nullptr)
-    , _keyboardUi(nullptr)
-    , _mouseUi(nullptr)
-    , _advancedUi(nullptr)
-    , _tempProfile(nullptr)
-    , _profile(nullptr)
-    , _isDefault(false)
-    , _previewedProperties(QHash<int, QVariant>())
-    , _delayedPreviewProperties(QHash<int, QVariant>())
-    , _delayedPreviewTimer(new QTimer(this))
-    , _colorDialog(nullptr)
-    , _buttonBox(nullptr)
-    , _fontDialog(nullptr)
+    , _tempProfile(new Profile{})
 {
+    _tempProfile->setHidden(true);
+
     setWindowTitle(i18n("Edit Profile"));
     setFaceType(KPageDialog::List);
 
@@ -89,8 +76,6 @@ EditProfileDialog::EditProfileDialog(QWidget *parent)
         }
     });
 
-    connect(_delayedPreviewTimer, &QTimer::timeout, this, &Konsole::EditProfileDialog::delayedPreviewActivate);
-
     // Set a fallback icon for non-plasma desktops as this dialog looks
     // terrible without all the icons on the left sidebar.  On GTK related
     // desktops, this dialog look good enough without installing
@@ -207,8 +192,6 @@ EditProfileDialog::EditProfileDialog(QWidget *parent)
     // not been updated since the last profile change and will need
     // to be refreshed when the user switches to them
     connect(this, &KPageDialog::currentPageChanged, this, &Konsole::EditProfileDialog::preparePage);
-
-    createTempProfile();
 }
 
 EditProfileDialog::~EditProfileDialog()
@@ -236,33 +219,29 @@ void EditProfileDialog::save()
         if (isNewProfile) {
             // New profile, we need to save it to disk, even if no settings
             // were changed and _tempProfile is empty
-            ProfileManager::instance()->changeProfile(_profile, _profile->setProperties());
+            ProfileManager::instance()->changeProfile(_profile, _profile->properties());
         }
         // no changes since last save
         return;
     }
 
-    ProfileManager::instance()->changeProfile(_profile, _tempProfile->setProperties());
+    ProfileManager::instance()->changeProfile(_profile, _tempProfile->properties());
 
     // ensure that these settings are not undone by a call
     // to unpreview()
-    QHashIterator<Profile::Property, QVariant> iter(_tempProfile->setProperties());
-    while (iter.hasNext()) {
-        iter.next();
-        _previewedProperties.remove(iter.key());
-    }
+    _previewedProperties.clear();
 
     // Update the default profile if needed
     if (defaultChanged) {
-        Q_ASSERT(_profile != ProfileManager::instance()->fallbackProfile());
+        Q_ASSERT(_profile != ProfileManager::instance()->builtinProfile());
 
         bool defaultChecked = _generalUi->setAsDefaultButton->isChecked();
-        Profile::Ptr newDefault = defaultChecked ? _profile : ProfileManager::instance()->fallbackProfile();
+        Profile::Ptr newDefault = defaultChecked ? _profile : ProfileManager::instance()->builtinProfile();
         ProfileManager::instance()->setDefaultProfile(newDefault);
         _isDefault = defaultChecked;
     }
 
-    createTempProfile();
+    resetTempProfile();
 
     _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 }
@@ -360,11 +339,14 @@ void EditProfileDialog::updateCaption(const Profile::Ptr &profile)
     if (group && group->profiles().count() > 1) {
         QString caption = groupProfileNames(group, MAX_GROUP_CAPTION_LENGTH);
         setWindowTitle(i18np("Editing profile: %2", "Editing %1 profiles: %2", group->profiles().count(), caption));
+        updateButtonApply();
     } else {
         if (_profileState == EditProfileDialog::NewProfile) {
             setWindowTitle(i18n("Create New Profile"));
+            _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
         } else {
             setWindowTitle(i18n("Edit Profile \"%1\"", profile->name()));
+            updateButtonApply();
         }
     }
 }
@@ -389,19 +371,12 @@ void EditProfileDialog::setProfile(const Konsole::Profile::Ptr &profile, EditPro
     }
     preparePage(currentPage());
 
-    if (_tempProfile) {
-        createTempProfile();
-    }
-}
-
-const Profile::Ptr EditProfileDialog::lookupProfile() const
-{
-    return _profile;
+    resetTempProfile();
 }
 
 const QString EditProfileDialog::currentColorSchemeName() const
 {
-    const QString &currentColorSchemeName = lookupProfile()->colorScheme();
+    const QString &currentColorSchemeName = _profile->colorScheme();
     return currentColorSchemeName;
 }
 
@@ -411,7 +386,7 @@ void EditProfileDialog::preparePage(KPageWidgetItem *current, KPageWidgetItem *b
     Q_ASSERT(current);
     Q_ASSERT(_pages.contains(current));
 
-    const Profile::Ptr profile = lookupProfile();
+    const Profile::Ptr profile = _profile;
     auto setupPage = _pages[current].setupPage;
     Q_ASSERT(profile);
     Q_ASSERT(setupPage);
@@ -486,6 +461,8 @@ void EditProfileDialog::setupGeneralPage(const Profile::Ptr &profile)
     } else {
         _generalUi->setAsDefaultButton->setText(i18n("Default profile"));
     }
+    _generalUi->semanticUpDown->setChecked(profile->semanticUpDown());
+    _generalUi->semanticInputClick->setChecked(profile->semanticInputClick());
 
     // signals and slots
     connect(_generalUi->dirSelectButton, &QToolButton::clicked, this, &Konsole::EditProfileDialog::selectInitialDir);
@@ -495,6 +472,8 @@ void EditProfileDialog::setupGeneralPage(const Profile::Ptr &profile)
     connect(_generalUi->initialDirEdit, &QLineEdit::textChanged, this, &Konsole::EditProfileDialog::initialDirChanged);
     connect(_generalUi->commandEdit, &QLineEdit::textChanged, this, &Konsole::EditProfileDialog::commandChanged);
     connect(_generalUi->environmentEditButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::showEnvironmentEditor);
+    connect(_generalUi->semanticUpDown, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::semanticUpDown);
+    connect(_generalUi->semanticInputClick, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::semanticInputClick);
 
     connect(_generalUi->terminalColumnsEntry, QOverload<int>::of(&QSpinBox::valueChanged), this, &Konsole::EditProfileDialog::terminalColumnsEntryChanged);
     connect(_generalUi->terminalRowsEntry, QOverload<int>::of(&QSpinBox::valueChanged), this, &Konsole::EditProfileDialog::terminalRowsEntryChanged);
@@ -504,12 +483,72 @@ void EditProfileDialog::setupGeneralPage(const Profile::Ptr &profile)
     });
 
     connect(_generalUi->setAsDefaultButton, &QAbstractButton::toggled, this, &Konsole::EditProfileDialog::updateButtonApply);
+    const ButtonGroupOptions semanticHints = {
+        _generalUi->semanticHints, // group
+        Profile::SemanticHints, // profileProperty
+        false, // preview
+        {
+            // buttons
+            {_generalUi->semanticHintsNever, Enum::HintsNever},
+            {_generalUi->semanticHintsURL, Enum::HintsURL},
+            {_generalUi->semanticHintsAlways, Enum::HintsAlways},
+        },
+    };
+    setupButtonGroup(semanticHints, profile);
+    const ButtonGroupOptions errorBars = {
+        _generalUi->errorBars, // group
+        Profile::ErrorBars, // profileProperty
+        false, // preview
+        {
+            // buttons
+            {_generalUi->errorBarsNever, Enum::HintsNever},
+            {_generalUi->errorBarsURL, Enum::HintsURL},
+            {_generalUi->errorBarsAlways, Enum::HintsAlways},
+        },
+    };
+    setupButtonGroup(errorBars, profile);
+    const ButtonGroupOptions errorBackground = {
+        _generalUi->errorBackground, // group
+        Profile::ErrorBackground, // profileProperty
+        false, // preview
+        {
+            // buttons
+            {_generalUi->errorBackgroundNever, Enum::HintsNever},
+            {_generalUi->errorBackgroundURL, Enum::HintsURL},
+            {_generalUi->errorBackgroundAlways, Enum::HintsAlways},
+        },
+    };
+    setupButtonGroup(errorBackground, profile);
+    const ButtonGroupOptions alternatingBars = {
+        _generalUi->alternatingBars, // group
+        Profile::AlternatingBars, // profileProperty
+        false, // preview
+        {
+            // buttons
+            {_generalUi->alternatingBarsNever, Enum::HintsNever},
+            {_generalUi->alternatingBarsURL, Enum::HintsURL},
+            {_generalUi->alternatingBarsAlways, Enum::HintsAlways},
+        },
+    };
+    setupButtonGroup(alternatingBars, profile);
+    const ButtonGroupOptions alternatingBackground = {
+        _generalUi->alternatingBackground, // group
+        Profile::AlternatingBackground, // profileProperty
+        false, // preview
+        {
+            // buttons
+            {_generalUi->alternatingBackgroundNever, Enum::HintsNever},
+            {_generalUi->alternatingBackgroundURL, Enum::HintsURL},
+            {_generalUi->alternatingBackgroundAlways, Enum::HintsAlways},
+        },
+    };
+    setupButtonGroup(alternatingBackground, profile);
 }
 
 void EditProfileDialog::showEnvironmentEditor()
 {
     bool ok;
-    const Profile::Ptr profile = lookupProfile();
+    const Profile::Ptr profile = _profile;
 
     QStringList currentEnvironment;
 
@@ -629,6 +668,16 @@ void EditProfileDialog::startInSameDir(bool sameDir)
     updateTempProfileProperty(Profile::StartInCurrentSessionDir, sameDir);
 }
 
+void EditProfileDialog::semanticUpDown(bool enable)
+{
+    updateTempProfileProperty(Profile::SemanticUpDown, enable);
+}
+
+void EditProfileDialog::semanticInputClick(bool enable)
+{
+    updateTempProfileProperty(Profile::SemanticInputClick, enable);
+}
+
 void EditProfileDialog::initialDirChanged(const QString &dir)
 {
     updateTempProfileProperty(Profile::Directory, dir);
@@ -676,30 +725,28 @@ void EditProfileDialog::setupAppearancePage(const Profile::Ptr &profile)
     _appearanceUi->removeColorSchemeButton->setEnabled(false);
     _appearanceUi->resetColorSchemeButton->setEnabled(false);
 
-    _appearanceUi->downloadColorSchemeButton->setConfigFile(QStringLiteral("konsole.knsrc"));
-
     // setup color list
     // select the colorScheme used in the current profile
     updateColorSchemeList(currentColorSchemeName());
 
     _appearanceUi->colorSchemeList->setMouseTracking(true);
-    _appearanceUi->colorSchemeList->installEventFilter(this);
     _appearanceUi->colorSchemeList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
 
     connect(_appearanceUi->colorSchemeList->selectionModel(), &QItemSelectionModel::selectionChanged, this, &Konsole::EditProfileDialog::colorSchemeSelected);
-    connect(_appearanceUi->colorSchemeList, &QListView::entered, this, &Konsole::EditProfileDialog::previewColorScheme);
+    connect(_appearanceUi->colorSchemeList, &QListView::activated, this, &Konsole::EditProfileDialog::previewColorScheme);
 
     updateColorSchemeButtons();
 
     connect(_appearanceUi->editColorSchemeButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::editColorScheme);
     connect(_appearanceUi->removeColorSchemeButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::removeColorScheme);
     connect(_appearanceUi->newColorSchemeButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::newColorScheme);
-    connect(_appearanceUi->downloadColorSchemeButton, &KNS3::Button::dialogFinished, this, &Konsole::EditProfileDialog::gotNewColorSchemes);
 
     connect(_appearanceUi->resetColorSchemeButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::resetColorScheme);
 
     connect(_appearanceUi->chooseFontButton, &QAbstractButton::clicked, this, &EditProfileDialog::showFontDialog);
 
+    connect(_appearanceUi->emojiFontPreview, &QAbstractButton::clicked, this, &EditProfileDialog::showEmojiFontDialog);
+
     // setup font preview
     const bool antialias = profile->antiAliasFonts();
 
@@ -709,6 +756,10 @@ void EditProfileDialog::setupAppearancePage(const Profile::Ptr &profile)
     _appearanceUi->fontPreview->setFont(profileFont);
     _appearanceUi->fontPreview->setText(QStringLiteral("%1 %2pt").arg(profileFont.family()).arg(profileFont.pointSize()));
 
+    QFont emojiFont = profile->emojiFont();
+    _appearanceUi->emojiFontPreview->setFont(profileFont);
+    _appearanceUi->emojiFontPreview->setText(QStringLiteral("%1 %2pt").arg(emojiFont.family()).arg(emojiFont.pointSize()));
+
     // setup font smoothing
     _appearanceUi->antialiasTextButton->setChecked(antialias);
     connect(_appearanceUi->antialiasTextButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::setAntialiasText);
@@ -720,9 +771,7 @@ void EditProfileDialog::setupAppearancePage(const Profile::Ptr &profile)
     connect(_appearanceUi->useFontLineCharactersButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::useFontLineCharacters);
 
     _mouseUi->enableMouseWheelZoomButton->setChecked(profile->mouseWheelZoomEnabled());
-    connect(_mouseUi->enableMouseWheelZoomButton, &QCheckBox::toggled, this,
-            &Konsole::EditProfileDialog::toggleMouseWheelZoom);
-
+    connect(_mouseUi->enableMouseWheelZoomButton, &QCheckBox::toggled, this, &Konsole::EditProfileDialog::toggleMouseWheelZoom);
 
     // cursor options
     _appearanceUi->enableBlinkingCursorButton->setChecked(profile->property<bool>(Profile::BlinkingCursorEnabled));
@@ -785,6 +834,43 @@ void EditProfileDialog::setupAppearancePage(const Profile::Ptr &profile)
 
     _appearanceUi->displayVerticalLineAtColumn->setValue(profile->verticalLineAtChar());
     connect(_appearanceUi->displayVerticalLineAtColumn, QOverload<int>::of(&QSpinBox::valueChanged), this, &EditProfileDialog::setVerticalLineColumn);
+
+#if KNEWSTUFF_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+    auto *getNewButton = new KNSWidgets::Button(this);
+    connect(getNewButton, &KNSWidgets::Button::dialogFinished, this, &Konsole::EditProfileDialog::gotNewColorSchemes);
+#else
+    auto *getNewButton = new KNS3::Button(this);
+    connect(getNewButton, &KNS3::Button::dialogFinished, this, &Konsole::EditProfileDialog::gotNewColorSchemes);
+#endif
+    getNewButton->setText(QStringLiteral("Get New..."));
+    getNewButton->setConfigFile(QStringLiteral("konsole.knsrc"));
+    _appearanceUi->colorSchemesBtnLayout->addWidget(getNewButton);
+
+    _appearanceUi->enableBidiRenderingButton->setChecked(profile->bidiRenderingEnabled());
+    connect(_appearanceUi->enableBidiRenderingButton, &QPushButton::toggled, this, &EditProfileDialog::togglebidiRendering);
+
+    _appearanceUi->enableBidiTableDirOverrideButton->setChecked(profile->property<bool>(Profile::BidiTableDirOverride));
+    connect(_appearanceUi->enableBidiTableDirOverrideButton, &QPushButton::toggled, this, &EditProfileDialog::togglebidiTableDirOverride);
+    _appearanceUi->enableBidiTableDirOverrideButton->setEnabled(profile->bidiRenderingEnabled());
+
+    _appearanceUi->bidiLineLTR->setChecked(profile->property<bool>(Profile::BidiLineLTR));
+    connect(_appearanceUi->bidiLineLTR, &QPushButton::toggled, this, &EditProfileDialog::togglebidiLineLTR);
+    _appearanceUi->bidiLineLTR->setEnabled(profile->bidiRenderingEnabled());
+
+    _appearanceUi->wordMode->setChecked(profile->property<bool>(Profile::WordMode));
+    connect(_appearanceUi->wordMode, &QPushButton::toggled, this, &EditProfileDialog::toggleWordMode);
+    _appearanceUi->wordModeAttr->setEnabled(profile->property<bool>(Profile::WordMode));
+    _appearanceUi->wordModeAscii->setEnabled(profile->property<bool>(Profile::WordMode));
+    _appearanceUi->wordModeBrahmic->setEnabled(profile->property<bool>(Profile::WordMode));
+
+    _appearanceUi->wordModeAttr->setChecked(profile->property<bool>(Profile::WordModeAttr));
+    connect(_appearanceUi->wordModeAttr, &QPushButton::toggled, this, &EditProfileDialog::toggleWordModeAttr);
+
+    _appearanceUi->wordModeAscii->setChecked(profile->property<bool>(Profile::WordModeAscii));
+    connect(_appearanceUi->wordModeAscii, &QPushButton::toggled, this, &EditProfileDialog::toggleWordModeAscii);
+
+    _appearanceUi->wordModeBrahmic->setChecked(profile->property<bool>(Profile::WordModeBrahmic));
+    connect(_appearanceUi->wordModeBrahmic, &QPushButton::toggled, this, &EditProfileDialog::toggleWordModeBrahmic);
 }
 
 void EditProfileDialog::setAntialiasText(bool enable)
@@ -993,19 +1079,6 @@ void EditProfileDialog::updateKeyBindingsList(const QString &selectKeyBindingsNa
     }
 }
 
-bool EditProfileDialog::eventFilter(QObject *watched, QEvent *event)
-{
-    if (watched == _appearanceUi->colorSchemeList && event->type() == QEvent::Leave) {
-        if (_tempProfile->isPropertySet(Profile::ColorScheme)) {
-            preview(Profile::ColorScheme, _tempProfile->colorScheme());
-        } else {
-            unpreview(Profile::ColorScheme);
-        }
-    }
-
-    return QDialog::eventFilter(watched, event);
-}
-
 QSize EditProfileDialog::sizeHint() const
 {
     QFontMetrics fm(font());
@@ -1020,86 +1093,47 @@ QSize EditProfileDialog::sizeHint() const
 
 void EditProfileDialog::unpreviewAll()
 {
-    _delayedPreviewTimer->stop();
-    _delayedPreviewProperties.clear();
-
-    QHash<Profile::Property, QVariant> map;
-    QHashIterator<int, QVariant> iter(_previewedProperties);
-    while (iter.hasNext()) {
-        iter.next();
-        map.insert(static_cast<Profile::Property>(iter.key()), iter.value());
-    }
-
     // undo any preview changes
-    if (!map.isEmpty()) {
-        ProfileManager::instance()->changeProfile(_profile, map, false);
+    if (!_previewedProperties.empty()) {
+        ProfileManager::instance()->changeProfile(_profile, _previewedProperties, false);
     }
 }
 
-void EditProfileDialog::unpreview(int property)
+void EditProfileDialog::unpreview(Profile::Property prop)
 {
-    _delayedPreviewProperties.remove(property);
-
-    if (!_previewedProperties.contains(property)) {
+    auto node = _previewedProperties.extract(prop);
+    if (!node) {
         return;
     }
 
-    QHash<Profile::Property, QVariant> map;
-    map.insert(static_cast<Profile::Property>(property), _previewedProperties[property]);
+    Profile::PropertyMap map;
+    map.insert(std::move(node));
     ProfileManager::instance()->changeProfile(_profile, map, false);
-
-    _previewedProperties.remove(property);
 }
 
-void EditProfileDialog::delayedPreview(int property, const QVariant &value)
+void EditProfileDialog::preview(Profile::Property prop, const QVariant &value)
 {
-    _delayedPreviewProperties.insert(property, value);
-
-    _delayedPreviewTimer->stop();
-    _delayedPreviewTimer->start(300);
-}
-
-void EditProfileDialog::delayedPreviewActivate()
-{
-    Q_ASSERT(qobject_cast<QTimer *>(sender()));
-
-    QMutableHashIterator<int, QVariant> iter(_delayedPreviewProperties);
-    if (iter.hasNext()) {
-        iter.next();
-        preview(iter.key(), iter.value());
-    }
-}
-
-void EditProfileDialog::preview(int property, const QVariant &value)
-{
-    QHash<Profile::Property, QVariant> map;
-    map.insert(static_cast<Profile::Property>(property), value);
-
-    _delayedPreviewProperties.remove(property);
-
-    const Profile::Ptr original = lookupProfile();
+    const Profile::Ptr original = _profile;
 
     // skip previews for profile groups if the profiles in the group
     // have conflicting original values for the property
     //
     // TODO - Save the original values for each profile and use to unpreview properties
     ProfileGroup::Ptr group = original->asGroup();
-    if (group && group->profiles().count() > 1 && original->property<QVariant>(static_cast<Profile::Property>(property)).isNull()) {
+    if (group && group->profiles().count() > 1 && original->property<QVariant>(prop).isNull()) {
         return;
     }
 
-    if (!_previewedProperties.contains(property)) {
-        _previewedProperties.insert(property, original->property<QVariant>(static_cast<Profile::Property>(property)));
-    }
+    _previewedProperties.insert({prop, original->property<QVariant>(prop)});
 
     // temporary change to color scheme
-    ProfileManager::instance()->changeProfile(_profile, map, false);
+    ProfileManager::instance()->changeProfile(_profile, {{prop, value}}, false);
 }
 
 void EditProfileDialog::previewColorScheme(const QModelIndex &index)
 {
     const QString &name = index.data(Qt::UserRole + 1).value<std::shared_ptr<const ColorScheme>>()->name();
-    delayedPreview(Profile::ColorScheme, name);
+    preview(Profile::ColorScheme, name);
 }
 
 void EditProfileDialog::showFontDialog()
@@ -1122,11 +1156,33 @@ void EditProfileDialog::showFontDialog()
             updateFontPreview(_profile->font());
         });
     }
-
     _fontDialog->setFont(_profile->font());
     _fontDialog->show();
 }
 
+void EditProfileDialog::showEmojiFontDialog()
+{
+    if (_emojiFontDialog == nullptr) {
+        _emojiFontDialog = new FontDialog(this, true, _profile->emojiFont());
+        _emojiFontDialog->setModal(true);
+        connect(_emojiFontDialog, &FontDialog::fontChanged, this, [this](const QFont &font) {
+            preview(Profile::EmojiFont, font);
+            updateEmojiFontPreview(font);
+        });
+        connect(_emojiFontDialog, &FontDialog::accepted, this, [this]() {
+            const QFont font = _emojiFontDialog->font();
+            preview(Profile::EmojiFont, font);
+            updateTempProfileProperty(Profile::EmojiFont, font);
+            updateEmojiFontPreview(font);
+        });
+        connect(_emojiFontDialog, &FontDialog::rejected, this, [this]() {
+            unpreview(Profile::EmojiFont);
+            updateEmojiFontPreview(_profile->emojiFont());
+        });
+    }
+    _emojiFontDialog->show();
+}
+
 void EditProfileDialog::updateFontPreview(QFont font)
 {
     bool aa = _profile->antiAliasFonts();
@@ -1136,6 +1192,15 @@ void EditProfileDialog::updateFontPreview(QFont font)
     _appearanceUi->fontPreview->setText(QStringLiteral("%1 %2pt").arg(font.family()).arg(font.pointSize()));
 }
 
+void EditProfileDialog::updateEmojiFontPreview(QFont font)
+{
+    bool aa = _profile->antiAliasFonts();
+    font.setStyleStrategy(aa ? QFont::PreferAntialias : QFont::NoAntialias);
+    QFont emojiFont = _profile->emojiFont();
+    _appearanceUi->emojiFontPreview->setFont(_profile->font());
+    _appearanceUi->emojiFontPreview->setText(QStringLiteral("%1 %2pt").arg(emojiFont.family()).arg(emojiFont.pointSize()));
+}
+
 void EditProfileDialog::removeColorScheme()
 {
     const QModelIndexList selected = _appearanceUi->colorSchemeList->selectionModel()->selectedIndexes();
@@ -1149,7 +1214,11 @@ void EditProfileDialog::removeColorScheme()
     }
 }
 
+#if KNEWSTUFF_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+void EditProfileDialog::gotNewColorSchemes(const QList<KNSCore::EntryInternal> &changedEntries)
+#else
 void EditProfileDialog::gotNewColorSchemes(const KNS3::Entry::List &changedEntries)
+#endif
 {
     int failures = 0;
     for (auto &entry : qAsConst(changedEntries)) {
@@ -1355,9 +1424,9 @@ void EditProfileDialog::updateTransparencyWarning()
     }
 }
 
-void EditProfileDialog::createTempProfile()
+void EditProfileDialog::resetTempProfile()
 {
-    _tempProfile = Profile::Ptr(new Profile);
+    _tempProfile->clear();
     _tempProfile->setHidden(true);
 }
 
@@ -1369,31 +1438,29 @@ void EditProfileDialog::updateTempProfileProperty(Profile::Property property, co
 
 void EditProfileDialog::updateButtonApply()
 {
-    bool userModified = false;
-
-    QHashIterator<Profile::Property, QVariant> iter(_tempProfile->setProperties());
-    while (iter.hasNext()) {
-        iter.next();
-
-        Profile::Property property = iter.key();
-        QVariant value = iter.value();
-
+    auto filterFunc = [this](const auto &keyValue) {
+        const auto &[property, value] = keyValue;
         // for previewed property
-        if (_previewedProperties.contains(static_cast<int>(property))) {
-            if (value != _previewedProperties.value(static_cast<int>(property))) {
-                userModified = true;
-                break;
+        auto it = _previewedProperties.find(property);
+        if (it != _previewedProperties.cend()) {
+            if (value != it->second) {
+                return true;
             }
             // for not-previewed property
             //
             // for the Profile::KeyBindings property, if it's set in the _tempProfile
             // then the user opened the edit key bindings dialog and clicked
             // OK, and could have add/removed a key bindings rule
-        } else if (property == Profile::KeyBindings || (value != _profile->property<QVariant>(property))) {
-            userModified = true;
-            break;
+        } else if (property == Profile::KeyBindings //
+                   || value != _profile->property<QVariant>(property)) {
+            return true;
         }
-    }
+
+        return false;
+    };
+
+    const Profile::PropertyMap &map = _tempProfile->properties();
+    bool userModified = std::any_of(map.cbegin(), map.cend(), filterFunc);
 
     if (_generalUi->setAsDefaultButton->isChecked() != _isDefault) {
         userModified = true;
@@ -1405,7 +1472,7 @@ void EditProfileDialog::updateButtonApply()
 void EditProfileDialog::setupKeyboardPage(const Profile::Ptr & /* profile */)
 {
     // setup translator list
-    updateKeyBindingsList(lookupProfile()->keyBindings());
+    updateKeyBindingsList(_profile->keyBindings());
 
     connect(_keyboardUi->keyBindingList->selectionModel(), &QItemSelectionModel::selectionChanged, this, &Konsole::EditProfileDialog::keyBindingSelected);
     connect(_keyboardUi->newKeyBindingsButton, &QPushButton::clicked, this, &Konsole::EditProfileDialog::newKeyBinding);
@@ -1470,7 +1537,7 @@ void EditProfileDialog::showKeyBindingEditor(bool isNewTranslator)
     editor->setModal(true);
 
     if (translator != nullptr) {
-        editor->setup(translator, lookupProfile()->keyBindings(), isNewTranslator);
+        editor->setup(translator, _profile->keyBindings(), isNewTranslator);
     }
 
     connect(editor, &Konsole::KeyBindingEditor::updateKeyBindingsListRequest, this, &Konsole::EditProfileDialog::updateKeyBindingsList);
@@ -1776,8 +1843,6 @@ void EditProfileDialog::setupAdvancedPage(const Profile::Ptr &profile)
     connect(_advancedUi->enableFlowControlButton, &QPushButton::toggled, this, &EditProfileDialog::toggleFlowControl);
     _appearanceUi->enableBlinkingCursorButton->setChecked(profile->property<bool>(Profile::BlinkingCursorEnabled));
     connect(_appearanceUi->enableBlinkingCursorButton, &QPushButton::toggled, this, &EditProfileDialog::toggleBlinkingCursor);
-    _advancedUi->enableBidiRenderingButton->setChecked(profile->property<bool>(Profile::BidiRenderingEnabled));
-    connect(_advancedUi->enableBidiRenderingButton, &QPushButton::toggled, this, &EditProfileDialog::togglebidiRendering);
     _advancedUi->enableReverseUrlHints->setChecked(profile->property<bool>(Profile::ReverseUrlHints));
     connect(_advancedUi->enableReverseUrlHints, &QPushButton::toggled, this, &EditProfileDialog::toggleReverseUrlHints);
 
@@ -1839,7 +1904,11 @@ int EditProfileDialog::maxSpinBoxWidth(const KPluralHandlingSpinBox *spinBox, co
     opt.frame = spinBox->hasFrame();
 
     const QSize hint(width, spinBox->sizeHint().height());
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    const QSize spinBoxSize = style()->sizeFromContents(QStyle::CT_SpinBox, &opt, hint, spinBox);
+#else
     const QSize spinBoxSize = style()->sizeFromContents(QStyle::CT_SpinBox, &opt, hint, spinBox).expandedTo(QApplication::globalStrut());
+#endif
 
     return spinBoxSize.width();
 }
@@ -1860,6 +1929,18 @@ void EditProfileDialog::wordCharactersChanged(const QString &text)
 void EditProfileDialog::togglebidiRendering(bool enable)
 {
     updateTempProfileProperty(Profile::BidiRenderingEnabled, enable);
+    _appearanceUi->enableBidiTableDirOverrideButton->setEnabled(enable);
+    _appearanceUi->bidiLineLTR->setEnabled(enable);
+}
+
+void EditProfileDialog::togglebidiTableDirOverride(bool enable)
+{
+    updateTempProfileProperty(Profile::BidiTableDirOverride, enable);
+}
+
+void EditProfileDialog::togglebidiLineLTR(bool enable)
+{
+    updateTempProfileProperty(Profile::BidiLineLTR, enable);
 }
 
 void EditProfileDialog::toggleUnderlineLinks(bool enable)
@@ -1970,3 +2051,26 @@ void EditProfileDialog::peekPrimaryKeySequenceChanged()
 {
     updateTempProfileProperty(Profile::PeekPrimaryKeySequence, _advancedUi->peekPrimaryWidget->keySequence().toString());
 }
+
+void EditProfileDialog::toggleWordMode(bool mode)
+{
+    updateTempProfileProperty(Profile::WordMode, mode);
+    _appearanceUi->wordModeAttr->setEnabled(mode);
+    _appearanceUi->wordModeAscii->setEnabled(mode);
+    _appearanceUi->wordModeBrahmic->setEnabled(mode);
+}
+
+void EditProfileDialog::toggleWordModeAttr(bool mode)
+{
+    updateTempProfileProperty(Profile::WordModeAttr, mode);
+}
+
+void EditProfileDialog::toggleWordModeAscii(bool mode)
+{
+    updateTempProfileProperty(Profile::WordModeAscii, mode);
+}
+
+void EditProfileDialog::toggleWordModeBrahmic(bool mode)
+{
+    updateTempProfileProperty(Profile::WordModeBrahmic, mode);
+}
